<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<gui name="practice"  version="4">

 <MenuBar>
   <Menu name="file" >
   <text>&amp;File</text>
   <Action name="practice_stop" />
  </Menu>

 <Menu name="view" >
   <text>&amp;View</text>
   <Action name="fullscreen" />
  </Menu>
 </MenuBar>

 <ToolBar noMerge="1" name="practiceToolBar" >
  <text>Practice Toolbar</text>
  <Action name="fullscreen" />
  <Action name="practice_stop" />
  <Action name="practice_export" />
 </ToolBar>
</gui>
