/*
    SPDX-FileCopyrightText: 2009 Daniel Laidig <d.laidig@gmx.de>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "abstractfrontend.h"

using namespace Practice;

AbstractFrontend::AbstractFrontend(QObject *parent)
    : QObject(parent)
{
}

#include "moc_abstractfrontend.cpp"
