/*
    view options for Parley
    SPDX-FileCopyrightText: 2005, 2007 Peter Hedlund <peter.hedlund@kdemail.net>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "viewoptions.h"

#include <QCheckBox>

ViewOptions::ViewOptions(QWidget *parent)
    : QWidget(parent)
{
    setupUi(this);
}

#include "moc_viewoptions.cpp"
